@@ -1,35 +1,37 @@
 # @package      hubzero-submit-common
 # @file         MessageCore.py
 # @author       Steven Clark <clarks@purdue.edu>
-# @copyright    Copyright (c) 2012-2014 HUBzero Foundation, LLC.
-# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
+# @copyright    Copyright (c) 2012-2015 HUBzero Foundation, LLC.
+# @license      http://opensource.org/licenses/MIT MIT
 #
-# Copyright (c) 2012-2014 HUBzero Foundation, LLC.
+# Copyright (c) 2012-2015 HUBzero Foundation, LLC.
 #
-# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the "Software"), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
+# furnished to do so, subject to the following conditions:
 #
-# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
-# software: you can redistribute it and/or modify it under the terms of
-# the GNU Lesser General Public License as published by the Free Software
-# Foundation, either version 3 of the License, or (at your option) any
-# later version.
+# The above copyright notice and this permission notice shall be included in
+# all copies or substantial portions of the Software.
 #
-# HUBzero is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+# THE SOFTWARE.
 #
 # HUBzero is a registered trademark of HUBzero Foundation, LLC.
 #
-
 from OpenSSL import SSL
 import sys
 import os
 import socket
 import time
+import math
 import traceback
 import logging
 from errno import EAGAIN,EWOULDBLOCK,EPIPE,ECONNRESET
@@ -48,23 +50,25 @@ class MessageCore:
                 sslCACertPath="",
                 reuseAddress=False,
                 blocking=True,
+                defaultBufferSize=None,
                 listenerHost="",
                 listenerPort=0,
                 listenerFile="",
                 repeatDelay=5):
-      self.logger       = logging.getLogger(__name__)
-      self.protocol     = protocol
-      self.family       = None
-      self.bindHost     = bindHost
-      self.bindPort     = bindPort
-      self.bindFile     = bindFile
-      self.bindLabel    = bindLabel
-      self.bindSocket   = None
-      self.listenerHost = listenerHost
-      self.listenerPort = listenerPort
-      self.listenerFile = listenerFile
-      self.repeatDelay  = repeatDelay
-      self.sslContext   = None
+      self.logger            = logging.getLogger(__name__)
+      self.protocol          = protocol
+      self.family            = None
+      self.bindHost          = bindHost
+      self.bindPort          = bindPort
+      self.bindFile          = bindFile
+      self.bindLabel         = bindLabel
+      self.bindSocket        = None
+      self.listenerHost      = listenerHost
+      self.listenerPort      = listenerPort
+      self.listenerFile      = listenerFile
+      self.repeatDelay       = repeatDelay
+      self.sslContext        = None
+      self.defaultBufferSize = defaultBufferSize
 
       if   bindPort > 0:
          self.family = socket.AF_INET
@@ -145,6 +149,10 @@ class MessageCore:
       return(isValid)
 
 
+   def getDefaultBufferSize(self):
+      return(self.defaultBufferSize)
+
+
    def getProtocol(self):
       return(self.protocol)
 
@@ -223,6 +231,7 @@ class MessageCore:
 
 
    def openListenerChannel(self,
+                           logErrorMessage=True,
                            recordTraceback=False):
       try:
          listenerChannel = self.getMessageSocket()
@@ -235,9 +244,16 @@ class MessageCore:
             listenerChannel.close()
          listenerChannel = None
          raise
+      except socket.gaierror,err:
+         if logErrorMessage:
+            self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (err.args[1])))
+         if listenerChannel:
+            listenerChannel.close()
+         listenerChannel = None
       except Exception,err:
          if err.args[0] == 111:
-            self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (err.args[1])))
+            if logErrorMessage:
+               self.logger.log(logging.ERROR,getLogMessage("openListenerChannel: %s" % (err.args[1])))
          else:
             if recordTraceback:
                self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
@@ -260,6 +276,17 @@ class MessageCore:
          listenerChannel.close()
 
 
+   def isBlockingChannel(self,
+                         channel):
+      timeout = channel.gettimeout()
+      if timeout == None:
+         blockingChannel = True
+      else:
+         blockingChannel = False
+
+      return(blockingChannel)
+
+
    def __receiveNonBlockingMessage(self,
                                    channel,
                                    bufferSize):
@@ -312,6 +339,8 @@ class MessageCore:
                                 channel,
                                 messageLength,
                                 bufferSize):
+      if messageLength <= 0:
+         self.logger.log(logging.ERROR,getLogMessage("Error in receiveBlockingMessage() %d message length" % (messageLength)))
       bytesRemaining = messageLength
       message = ""
 
@@ -339,11 +368,16 @@ class MessageCore:
                       channel,
                       messageLength,
                       bufferSize=128):
-      timeout = channel.gettimeout()
-      if timeout == None:
+      if self.isBlockingChannel(channel):
+#        self.logger.log(logging.DEBUG,getLogMessage("MessageCore:receiveMessage(blocked,%d): %d" % \
+#                                                                 (bufferSize,channel.fileno())))
          message = self.__receiveBlockingMessage(channel,messageLength,bufferSize)
+#        self.logger.log(logging.DEBUG,getLogMessage("MessageCore:receiveMessage(blocked): %d %s" % \
+#                                                                    (channel.fileno(),message)))
       else:
          message = self.__receiveNonBlockingMessage(channel,bufferSize)
+#        self.logger.log(logging.DEBUG,getLogMessage("MessageCore:receiveMessage(nonblocked): %d %s" % \
+#                                                                    (channel.fileno(),message)))
 
       return(message)
 
@@ -421,11 +455,13 @@ class MessageCore:
                    message,
                    fixedBufferSize=0):
       if channel:
-#        log("MessageCore:sendMessage(%s)" % (message))
-         timeout = channel.gettimeout()
-         if timeout == None:
+         if self.isBlockingChannel(channel):
+#           self.logger.log(logging.DEBUG,getLogMessage("MessageCore:sendMessage(blocked,%d): %d %s" % \
+#                                                       (fixedBufferSize,channel.fileno(),message)))
             transmittedLength = self.__sendBlockingMessage(channel,message,fixedBufferSize)
          else:
+#           self.logger.log(logging.DEBUG,getLogMessage("MessageCore:sendMessage(nonblocked): %d %s" % \
+#                                                              (channel.fileno(),message)))
             transmittedLength = self.__sendNonBlockingMessage(channel,message)
       else:
          transmittedLength = 0
@@ -433,116 +469,82 @@ class MessageCore:
       return(transmittedLength)
 
 
-   def requestMessageResponse(self,
-                              message,
-                              messageBufferSize,
-                              responseBufferSize,
-                              recordTraceback=False):
-      nTry = 0
-      delay = 0
-      posted = False
-      messageResponseSocket = None
-      while not posted:
-         time.sleep(delay)
-         try:
-            nTry += 1
-            messageResponseSocket = self.getMessageSocket()
-            if   self.family == socket.AF_INET:
-               messageResponseSocket.connect((self.listenerHost,self.listenerPort))
-            elif self.family == socket.AF_UNIX:
-               messageResponseSocket.connect(self.listenerFile)
-            if self.__sendBlockingMessage(messageResponseSocket,message,messageBufferSize) > 0:
-               response = self.__receiveBlockingMessage(messageResponseSocket,responseBufferSize,128)
-               if response != "":
-                  posted = True
-         except:
-            if recordTraceback:
-               self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
+   def __acceptHandshake(self,
+                         channel,
+                         bufferSize):
+      valid = False
+      message = "SUBMIT %d" % (bufferSize)
+      message = message.ljust(32,' ')
+      reply = ""
 
-         if messageResponseSocket:
-            messageResponseSocket.close()
-         delay = self.repeatDelay
+      try:
+         # Write the message.
+         nSent = self.sendMessage(channel,message)
 
-      return(nTry,response)
-
-
-   def requestMessageVariableResponse(self,
-                                      message,
-                                      messageBufferSize,
-                                      responseBufferSize,
-                                      recordTraceback=False):
-      response = ""
-      nTry = 0
-      delay = 0
-      posted = False
-      messageResponseSocket = None
-      while not posted:
-         time.sleep(delay)
-         try:
-            nTry += 1
-            messageResponseSocket = self.getMessageSocket()
-            if   self.family == socket.AF_INET:
-               messageResponseSocket.connect((self.listenerHost,self.listenerPort))
-            elif self.family == socket.AF_UNIX:
-               messageResponseSocket.connect(self.listenerFile)
-            if self.__sendBlockingMessage(messageResponseSocket,message,messageBufferSize) > 0:
-               responseHeader = self.__receiveBlockingMessage(messageResponseSocket,responseBufferSize,responseBufferSize)
-               if responseHeader != "":
-                  responseLength = responseHeader.strip()
-                  if int(responseLength) > 0:
-                     response = self.__receiveBlockingMessage(messageResponseSocket,int(responseLength),int(responseLength))
-                     if response != "":
-                        posted = True
+         if nSent > 0:
+            # Expect the same message back.
+            reply = self.receiveMessage(channel,nSent,nSent)
+            if len(reply) == len(message):
+               if reply[0:7] == message[0:7]:
+                  valid = True
+                  if reply.strip()[-1] == '1':
+                     channel.setblocking(1)
                   else:
-                     posted = True
-         except:
-            if recordTraceback:
-               self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
+                     channel.setblocking(0)
+      except Exception, err:
+         self.logger.log(logging.ERROR,getLogMessage("Connection acceptHandshake failed.  Protocol mismatch?"))
+         self.logger.log(logging.ERROR,getLogMessage("acceptHandshake(%s): %s" % (message.strip(),str(reply).strip())))
+         self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
 
-         if messageResponseSocket:
-            messageResponseSocket.close()
-         delay = self.repeatDelay
-
-      return(nTry,response)
+      return(valid)
 
 
-   def requestMessageTimestampResponse(self,
-                                       message,
-                                       messageBufferSize,
-                                       responseBufferSize,
-                                       recordTraceback=False):
-      response = ""
-      nTry = 0
-      delay = 0
-      posted = False
-      messageResponseSocket = None
-      while not posted:
-         time.sleep(delay)
-         try:
-            nTry += 1
-            messageResponseSocket = self.getMessageSocket()
-            if   self.family == socket.AF_INET:
-               messageResponseSocket.connect((self.listenerHost,self.listenerPort))
-            elif self.family == socket.AF_UNIX:
-               messageResponseSocket.connect(self.listenerFile)
-            if self.__sendBlockingMessage(messageResponseSocket,message,messageBufferSize) > 0:
-               responseHeader = self.__receiveBlockingMessage(messageResponseSocket,responseBufferSize,responseBufferSize)
-               if responseHeader != "":
-                  responseLength,responseTimestamp = responseHeader.strip().split()
-                  if int(responseLength) > 0:
-                     response = self.__receiveBlockingMessage(messageResponseSocket,int(responseLength),int(responseLength))
-                     if response != "":
-                        posted = True
+   def acceptHandshake(self,
+                       channel):
+      valid = self.__acceptHandshake(channel,self.defaultBufferSize)
+
+      return(valid)
+
+
+   def __initiateHandshake(self,
+                           channel,
+                           blocking):
+      valid = False
+      if blocking:
+         message = "SUBMIT 1"
+      else:
+         message = "SUBMIT 0"
+      message = message.ljust(32,' ')
+      reply = ""
+
+      try:
+         # Write the message.
+         nSent = self.sendMessage(channel,message)
+
+         if nSent > 0:
+            # Expect the same message back.
+            reply = self.receiveMessage(channel,nSent,nSent)
+            if len(reply) == len(message):
+               if reply[0:7] == message[0:7]:
+                  valid = True
+                  if blocking:
+                     channel.setblocking(1)
                   else:
-                     posted = True
-         except:
-            if recordTraceback:
-               self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
+                     channel.setblocking(0)
+                  self.defaultBufferSize = int(reply.split()[-1])
+      except Exception, err:
+         self.logger.log(logging.ERROR,getLogMessage("Connection initiateHandshake failed.  Protocol mismatch?"))
+         self.logger.log(logging.ERROR,getLogMessage("initiateHandshake(%s): %s" % (message.strip(),str(reply).strip())))
+         self.logger.log(logging.ERROR,getLogMessage(traceback.format_exc()))
 
-         if messageResponseSocket:
-            messageResponseSocket.close()
-         delay = self.repeatDelay
+      return(valid)
 
-      return(nTry,response,responseTimestamp)
+
+   def initiateHandshake(self,
+                         channel,
+                         blocking=False):
+      valid = self.__initiateHandshake(channel,blocking)
+
+      return(valid)
 
 
